/*      
        alph: Piper program implementing and analysing traditional and
        historic c(ai)phers designed to be used in conjuction with other
        programs.

        Copyright (C) 2004  Corcalciuc V. Horia

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

        Dedicated to Rudolf.
*/


/* System */
#include <stdio.h>
#include <string.h>

/* Our own. */
#include "alphCRYPT.h"
#include "alphan.h"
#include "config.h"

void 
usage ( char *copts[], char *copts_desc[], char tgg[], char *tgg_desc[] ) 
{
  int forw;

  printf ("\n\t\t%s v%s\n", PACKAGE_NAME, PACKAGE_VERSION);
  printf ("\n Ciphers:\n");
  for (forw=0; forw < 10; forw++)
    printf ("\t%s \t%s\n", copts[forw], copts_desc[forw]);
  printf("\n Options:\n");
  for (forw=0; forw < 3; forw++)
    if (forw!=2)
      printf ("\t-%c \t\t%s\n", tgg[forw], tgg_desc[forw]);
    else if(forw==2)
      printf("\t-%c n \t\t%s\n", tgg[forw], tgg_desc[forw]);
  printf("\t-h, --help\t%s\n", "this usage info");
  printf ("\n\t(c) Corcalciuc V. Horia - rain@sevenrains.ro\n\n");
  
  exit(1);
}

int main(argc, argv)
int argc;
char **argv;
{

  int topt=argc;
  int optr;
  int forw;
  int xtime;
  char *key[5];

  char *copts[] = { "--CAESAR",
                    "--ATBASH", 
                    "--ROT13", 
                    "--VIGENERE",
                    "--VERNAM",
                    "--PLAYFAIR",
                    "--BIFID",
                    "--ENIGMA",
                    "--MORSE",
						  "--TWOSQUARE"
                  };

  char cid[] = { 'c', 'a', 'r', 'v', 'u', 'p', 'b', 'e', 'm', 't' };
  char *copts_desc[] = { "",
                         "",
                         "",
                         "ALPHA KEY",
                         "FILE",
                         "ALPHA KEY",
                         "ALPHA KEY",
                         "ROTORS REFLECTOR RING",
                         "",
                         "ALPHA KEY ALPHA KEY"
                       };
  char tgg[] = { 'e', 'd', 's' };
  char *tgg_desc[] = { "encrypt the stream (def.)",
                       "decrypt the stream",
                       "freq subtitute n times (def: 1)"
                     };
  typedef enum { false = 0, true } flip;
  flip dsmash = false;
  char *indata;
  int incr;
  FILE *fp;

  for (optr=argc-1; optr >= 0; optr--)
    if (strlen (argv[optr]) < 7)
      if (strstr(argv[optr], "-h") || strstr(argv[optr], "--help"))
        {
            usage(copts, copts_desc, tgg, tgg_desc);
            return 1;
        }

  indata = calloc (BUFSIZ, sizeof(char));

  fflush(stdin);
  for (optr = 0; optr <= BUFSIZ; optr++)
    if (!feof (stdin))
      indata[optr] = fgetc (stdin);
    else
      {
        indata[optr -1] = '\0';
        break;
      }

  while (argc-->0)
    {
      for (optr=0; optr<10; optr++)
        {
          if (!strcmp(copts[optr],argv[argc]-'\0'))
            switch (cid[optr])
              {
                case 'c':
                  caesar (indata);
                  break;
                case 'a':
                  atbash (indata);
                  break;
                case 'r':
                  rot13 (indata);
                  break;
                case 'v':
                  if (argv[argc+1]==NULL)
                    return KEYB;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  strcpy (key[0], argv[argc+1]);
                  vigenere (indata, key[0]);
                  break;
                case 'u':
                  if (argv[argc+1]==NULL)
                    return KEYB;
                  incr = 0;
                  key[0] = (char *) calloc (strlen(indata), sizeof(char));
                  if ((fp = fopen (argv[argc+1], "r")) == NULL)
                    exit(1);
                  else
                    while ((forw = fgetc(fp)) != EOF && incr < strlen(indata))
                      if (isalpha (forw))
                        key[0][incr++] = forw;
                  vernam (indata, key[0]);
                  break;
                case 'p':
                  if (argv[argc+1]==NULL)
                    return KEYB;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  playfair (indata, key[0]);
                  break;
                case 'b':
                  if (argv[argc+1]==NULL)
                    return KEYB;
                  if ( argc+1 > topt || !isalpha(argv[argc+1][0]))
                    return KEYB;
                  key[0] = (char *) calloc (strlen(argv[argc+1]), sizeof(char));
                  for (incr=0; incr <= strlen(argv[argc+1]); incr++)
                    key[0][incr]=argv[argc+1][incr];
                  bifid (indata, key[0]);
                  break;
                case 'e':
                  for (forw=1; forw<4; forw++)
                  {
                    if ( argc+forw > topt )
                        return KEYB;
                    if ( forw==1 )
                      {
                        if ( !isdigit(argv[argc+forw][0]) )
                          return KEYB;
                      }
                    else
                      {
			            if ( !isalpha(argv[argc+forw][0]) )
                          return KEYB;
                      }
			
                    key[forw-1] = (char *) calloc (strlen(argv[argc+forw]), sizeof(char));
                    strcpy ( key[forw-1], argv[argc+forw] );
                  }
                  enigma ( indata, key[0], key[1], key[2] );
                  break;
                case 'm':
                  morse(indata);
                  break;
					 case 't':
                  for (forw=1; forw<3; forw++)
                  {
                    if ( argc+forw > topt )
                        return KEYB;
			            if ( !isalpha(argv[argc+forw][0]) )
                          return KEYB;
                    key[forw-1] = (char *) calloc (strlen(argv[argc+forw]), sizeof(char));
                    strcpy ( key[forw-1], argv[argc+forw] );
                  }
						twosquare ( indata, key[0], key[1] );
						break;
              }
          else if (argv[argc][1]==tgg[optr])
            if (argv[argc][0]=='-')
              {
                switch (tgg[optr])
                  {
                    case 'e':
                      al_dec=false;
                      break;
                    case 'd':
                      al_dec=true;
                      break;
                    case 's':
                      if (strchr(indata, ' '))
                        {
                          dsmash=true;
                          if(argv[argc+1]!=NULL)
                            {
                              xtime=atoi(argv[argc+1]);
                              if (xtime<0 || xtime>9)
                                return ENTO;
                            }
                          else
                              xtime=true;
                        }
                      else
                        return ENTO;
                      break;
                  }
              }
        }
    }

  if (dsmash)
    {
      do {
        indata=frq(indata);
      } while(--xtime>0);
      printf("%s", indata);
    }
  else
    printf("%s", indata);

  exit(0);

}

